using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Text.RegularExpressions;

namespace howto_custom_sort_listbox
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Display items sorted in numeric order.
        private void Form1_Load(object sender, EventArgs e)
        {
            lstAlphabetic.Sorted = true;
            cboAlphabetic.Sorted = true;

            SortListBoxItems(lstNumeric);
            SortComboBoxItems(cboNumeric);

            lstAlphabetic.SelectedIndex = -1;
            lstNumeric.SelectedIndex = -1;
            cboAlphabetic.SelectedIndex = -1;
            cboNumeric.SelectedIndex = -1;
        }

        // Sort the items in the ListBox by their contained numeric values.
        private void SortListBoxItems(ListBox lst)
        {
            // Get the original items as an array.
            int num_items = lst.Items.Count;
            object[] items = new object[num_items];
            lst.Items.CopyTo(items, 0);

            // Sort them by their contained numeric values.
            items = SortNumericItems(items);

            // Display the results.
            lst.Sorted = false;
            lst.DataSource = items;
        }

        // Sort the items in the ComboBox by their contained numeric values.
        private void SortComboBoxItems(ComboBox cbo)
        {
            // Get the original items as an array.
            int num_items = cbo.Items.Count;
            object[] items = new object[num_items];
            cbo.Items.CopyTo(items, 0);

            // Sort them by their contained numeric values.
            items = SortNumericItems(items);

            // Display the results.
            cbo.Sorted = false;
            cbo.DataSource = items;
        }

        // Sort items by contained numeric values.
        private object[] SortNumericItems(object[] items)
        {
            // Get the numeric values of the items.
            int num_items = items.Length;
            const string float_pattern = @"-?\d+\.?\d*";
            double[] values = new double[num_items];
            for (int i = 0; i < num_items; i++)
            {
                string match = Regex.Match(
                    items[i].ToString(), float_pattern).Value;
                double value;
                if (!double.TryParse(match, out value))
                    value = double.MinValue;
                values[i] = value;

                //Console.WriteLine(value + ": " + items[i].ToString());
            }

            // Sort the items array using the keys to determine order.
            Array.Sort(values, items);
            return items;
        }
    }
}
